/* Trencherous Skies
   2D Game Project

   Lawrence Chan
   Patrick Habib
   CIS 487/587
*/

// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  

#define INITGUID // if not done elsewhere

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <objbase.h>
#include <iostream> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>

#include <ddraw.h>     // directX includes
#include <dinput.h>
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>

#include "T3DLIB1.h" // game library includes
#include "T3DLIB3.h" // game library includes

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINCLASS1"

// default screen size
#define SCREEN_WIDTH		 640  // size of screen
#define SCREEN_HEIGHT		 480
#define SCREEN_BPP			 8    // bits per pixel
#define MAX_SCREEN_HEIGHT	 415 //right above the scoreboard

#define BITMAP_ID            0x4D42 // universal id for a bitmap
#define MAX_COLORS_PALETTE   256

#define NUM_SHOTS			 16 //total shots for player
#define NUM_ROCKETS			 3 //total homing rockets for fire ship
#define NUM_ICICLES			 2 //total lasers for ice ship
#define NUM_VIRUSES			 5 //total viruses for eyeball
#define NUM_SEGMENTS		 5 //total segments for eyeball
#define NUM_FISH			 10 //total fishes to appear
#define NUM_SQUID			 10 //total squids to appear
#define NUM_SHIP			 10 //total normal ships to appear
#define MAX_EXPLOSIONS		 500 //total number of explosions to appear
#define EXPLODE_SMALL_X		 25 //x size of a small explosion
#define EXPLODE_SMALL_Y		 32 //y size of a small explosion
#define EXPLODE_BIG_X		 56 //x size of a big explosion
#define EXPLODE_BIG_Y		 72 //y size of a big explosion

#define OFFSCREEN_X			 640 //offscreen positions--the lower right
#define	OFFSCREEN_Y			 480 //corner of the screen

#define BOSS_HP				 300 //boss hp
#define NORMAL_HP			 25 //normal enemy hp

#define WALL_ANIMATION_COLOR 29

//wave numbers
#define WAVE_BLUE_FISH		 0
#define WAVE_RED_FISH		 1
#define WAVE_NORMAL_SHIP	 2
#define WAVE_SQUID			 3
#define WAVE_DUAL_SHIPS		 4
#define WAVE_CRYSTALS		 5
#define WAVE_EYEBALL		 6
#define ENDING_TIME			 7

// TYPES ///////////////////////////////////////////////////

typedef struct PROJ_TYP
{
int state;      // state 0 off, 1 on
float x,y;      // position
float xv, yv;   // velocity

} PROJECTILE, *PROJECTILE_PTR;

// PROTOTYPES /////////////////////////////////////////////

// game console
int Game_Init(void *parms=NULL, int num_parms = 0);
int Game_Shutdown(void *parms=NULL, int num_parms = 0);
int Game_Main(void *parms=NULL,  int num_parms = 0);

// GLOBALS ////////////////////////////////////////////////

// windows vars
HWND      main_window_handle = NULL; // globally track main window
int       window_closed      = 0;    // tracks if window is closed
HINSTANCE main_instance      = NULL; // globally track hinstance

char buffer[80];                // used to print text

// directinput globals
LPDIRECTINPUT8        lpdi      = NULL;    // dinput object
LPDIRECTINPUTDEVICE8  lpdikey   = NULL;    // dinput keyboard
LPDIRECTINPUTDEVICE8  lpdimouse = NULL;    // dinput mouse
LPDIRECTINPUTDEVICE8  lpdijoy   = NULL;    // dinput joystick 
GUID                 joystickGUID;        // guid for main joystick
char                 joyname[80];         // name of joystick

// these contain the target records for all di input packets
UCHAR        keyboard_state[256]; // contains keyboard state table
DIMOUSESTATE mouse_state;         // contains state of mouse
DIJOYSTATE   joy_state;           // contains state of joystick

// demo globals

//Player BOB
BOB          ship; // the player ship
BOB			 normal_shot, heat_wave, ice_pillar; //weapons

//enemy weapons
BOB			 rocket; //for fire ship
BOB			 icicle; //for ice ship
BOB			 virus; //for eyeball

//enemies
BOB          fire_ship;              
BOB			 ice_ship;
BOB			 eyeball;
BOB			 segments[NUM_SEGMENTS];
BOB			 blue_fish[NUM_FISH];
BOB			 red_fish[NUM_FISH];
BOB			 normal_ship[NUM_SHIP];
BOB			 squid[NUM_SQUID];
BOB			 crystal1, crystal2, crystal3;

//explosions
BOB			 explosion[MAX_EXPLOSIONS];

//projectiles
PROJECTILE	 missiles[NUM_SHOTS]; //array of missiles
PROJECTILE	 rockets[NUM_ROCKETS]; //array of rockets
PROJECTILE	 icicles[NUM_ICICLES]; //array of lasers
PROJECTILE	 viruses[NUM_VIRUSES]; //array of viruses

//background images
BITMAP_IMAGE level1; //the background for level 1
BITMAP_IMAGE level2; //the background for level 2
BITMAP_IMAGE titlescreen; //the main title screen
BITMAP_IMAGE gameoverscreen; //the game over screen
BITMAP_IMAGE keyboard; //directions for keyboard position
BITMAP_IMAGE story1; //story sequences
BITMAP_IMAGE story2;
BITMAP_IMAGE story3;
BITMAP_IMAGE story4;
BITMAP_IMAGE story5;
BITMAP_IMAGE directions; //directions for game play
BITMAP_IMAGE ending1; //ending sequences
BITMAP_IMAGE ending2;
BITMAP_IMAGE credits;

//global variables for weapon equipped, total ships, scoreboard, and various flags
int weapon = 0; int wspeed = 10; int num_ships = 5;
int points_awarded = 1;
int num_wave = 0; //determines which enemy wave to appear
int scoreboard = 0;
int storycounter = 0; //tells which story slide to show
int endingcounter = 0; //tells which ending slide to show

//sound ids and music
int explode_ids[MAX_EXPLOSIONS]; //explosion sound
int beam_ids[NUM_SHOTS]; //normal shot sound
int ice_ids[NUM_SHOTS]; //ice pillar sound
int fire_ids[NUM_SHOTS]; //heat wave sound
int rocket_ids[NUM_ROCKETS]; //homing rocket sound
int laser_ids[NUM_ICICLES]; //laser sound
int generate_ids[NUM_VIRUSES]; //virus generation sound
int intromusic; //introduction and story music
int endingmusic; //ending and credits music
int lvl1music; //level 1 music
int lvl2music; //level 2 music
int music;
int shield; //sound for when shield is low
int continuesound; //sound when continuing story/ending

//flags for game control
bool ship_defeated = false; //flag for 2nd boss (eyeball)
bool storytime = false; //flag for story time
bool gamestart = false; //flag for starting the game
bool gameover = false; //game over flag
bool gameend = false; //ending flag
bool key_space_up = false; //flag for storytelling


// PROTOTYPES //////////////////////////////////////////////

//AI functions
void Blue_fish_AI(void); //blue fish AI
void Red_fish_AI(void); //red fish AI
void Normal_ship_AI(void); //normal ship AI
void Squid_AI(void); //squid AI
void Fire_ship_AI(void); //fire ship AI
void Ice_ship_AI(void); //ice ship AI
void Eyeball_AI(void); //eyeball AI
void Segment_AI(void); //segment AI

// missile interface
void Init_Projectiles(void); //initialize all projectiles
void Move_Projectiles(void); //move all projectiles
void Draw_Projectiles(void); //draw all projectiles
void Fire_Projectile(int angle, float vel); //fire player projectile
void Fire_Enemy1_Projectile(int angle, float vel); //fire ship projectile
void Fire_Enemy2_Projectile(int angle, float vel); //ice ship projectile
void Fire_Enemy3_Projectile(int angle, float vel); //eyeball projectile

//Damage Functions
void Player_Defeated();
void Explode_Area(int x_pos, int y_pos, int x_size, int y_size);
void Check_Damage(BOB& target, PROJECTILE& missile, int weakness);
void Check_Player_Damage(PROJECTILE& missile);
void Check_Player_Enemy_Collision(BOB enemy, bool &hit);

//Sound functions
void Explosion_Sound(void);
void Beam_Sound(void);
void Ice_Sound(void);
void Fire_Sound(void);
void Rocket_Sound(void);
void Laser_Sound(void);
void Generate_Sound(void);

//Delete BOB arrays
void Delete_Segments(void);
void Delete_Explosions(void);
void Delete_Blue_Fishes(void);
void Delete_Red_Fishes(void);
void Delete_Normal_Ship(void);
void Delete_Squids(void);

//Reset game function
void ResetGame(void);

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT		ps;		// used in WM_PAINT
HDC				hdc;	// handle to a device context
char buffer[80];        // used to print strings

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here
        // return success
		return(0);
		} break;
   
	case WM_PAINT: 
		{
		// simply validate the window 
   	    hdc = BeginPaint(hwnd,&ps);	 
        
        // end painting
        EndPaint(hwnd,&ps);

        // return success
		return(0);
   		} break;

	case WM_DESTROY: 
		{

		// kill the application, this sends a WM_QUIT message 
		PostQuitMessage(0);

        // return success
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{

WNDCLASSEX winclass; // this will hold the class we create
HWND	   hwnd;	 // generic window handle
MSG		   msg;		 // generic message
HDC        hdc;      // graphics device context

// first fill in the window class stucture
winclass.cbSize         = sizeof(WNDCLASSEX);
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW); 
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= NULL;
winclass.lpszClassName	= WINDOW_CLASS_NAME;
winclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);

// save hinstance in global
main_instance = hinstance;

// register the window class
if (!RegisterClassEx(&winclass))
	return(0);

// create the window
if (!(hwnd = CreateWindowEx(NULL,                  // extended style
                            WINDOW_CLASS_NAME,     // class
						    "DirectInput Keyboard Demo", // title
						    WS_POPUP | WS_VISIBLE,
					 	    0,0,	  // initial x,y
						    SCREEN_WIDTH,SCREEN_HEIGHT,  // initial width, height
						    NULL,	  // handle to parent 
						    NULL,	  // handle to menu
						    hinstance,// instance of this application
						    NULL)))	// extra creation parms
return(0);

// save main window handle
main_window_handle = hwnd;

// initialize game here
Game_Init();

// enter main event loop
while(TRUE)
	{
    // test if there is a message in queue, if so get it
	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
	   { 
	   // test if this is a quit
       if (msg.message == WM_QUIT)
           break;
	
	   // translate any accelerator keys
	   TranslateMessage(&msg);

	   // send the message to the window proc
	   DispatchMessage(&msg);
	   } // end if
    
       // main game processing goes here
       Game_Main();
       
	} // end while

// closedown game here
Game_Shutdown();

// return to Windows like this
return(msg.wParam);

} // end WinMain

// GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

int Game_Init(void *parms,  int num_parms)
{
// this function is where you do all the initialization 
// for your game

int index;         // looping var
char filename[80]; // used to build up files names

// initialize directdraw
DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);
// initilize DirectSound
DSound_Init();
// initialize directmusic
DMusic_Init();

#if 0 // directinput7 method

// first create the direct input object
if (DirectInputCreateEx(main_instance,DIRECTINPUT_VERSION,IID_IDirectInput7, 
	(void **)&lpdi,NULL)!=DI_OK)
   return(0);

// create a keyboard device  //////////////////////////////////
if (lpdi->CreateDeviceEx(GUID_SysKeyboard, IID_IDirectInputDevice7, 
	(void **)&lpdikey, NULL)!=DI_OK)
   return(0);

#endif


// first create the direct input object
if (DirectInput8Create(main_instance,DIRECTINPUT_VERSION,IID_IDirectInput8, 
	(void **)&lpdi,NULL)!=DI_OK)
   return(0);

// create a keyboard device  //////////////////////////////////
if (lpdi->CreateDevice(GUID_SysKeyboard, &lpdikey, NULL)!=DI_OK)
   return(0);


// set cooperation level
if (lpdikey->SetCooperativeLevel(main_window_handle, 
                 DISCL_NONEXCLUSIVE | DISCL_BACKGROUND)!=DI_OK)
    return(0);

// set data format
if (lpdikey->SetDataFormat(&c_dfDIKeyboard)!=DI_OK)
   return(0);

// acquire the keyboard
if (lpdikey->Acquire()!=DI_OK)
   return(0);

///////////////////////////////////////////////////////////

// load midi file
intromusic = DMusic_Load_MIDI("AUDIO/INTRO.MID");

// load midi file
lvl1music = DMusic_Load_MIDI("AUDIO/LEVEL1.MID");

// load midi file
lvl2music = DMusic_Load_MIDI("AUDIO/LEVEL2.MID");

// load midi file
endingmusic = DMusic_Load_MIDI("AUDIO/ENDING.MID");

//Load background bitmap images////////////////////////////////////

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/LEVEL1.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&level1, 0,0, 640, 480);
Load_Image_Bitmap(&level1,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/LEVEL2.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&level2, 0,0, 640, 480);
Load_Image_Bitmap(&level2,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/TITLE.BMP");

// set the palette to background image palette
Set_Palette(bitmap8bit.palette);

// create and load the level1 bitmap image
Create_Bitmap(&titlescreen, 0,0, 640, 480);
Load_Image_Bitmap(&titlescreen,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/GAMEOVER.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&gameoverscreen, 0,0, 640, 480);
Load_Image_Bitmap(&gameoverscreen,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/KEYBOARD.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&keyboard, 0,0, 640, 480);
Load_Image_Bitmap(&keyboard,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/STORY1.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&story1, 0,0, 640, 480);
Load_Image_Bitmap(&story1,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/STORY2.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&story2, 0,0, 640, 480);
Load_Image_Bitmap(&story2,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/STORY3.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&story3, 0,0, 640, 480);
Load_Image_Bitmap(&story3,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/STORY4.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&story4, 0,0, 640, 480);
Load_Image_Bitmap(&story4,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/STORY5.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&story5, 0,0, 640, 480);
Load_Image_Bitmap(&story5,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/DIRECTIONS.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&directions, 0,0, 640, 480);
Load_Image_Bitmap(&directions,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/ENDING1.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&ending1, 0,0, 640, 480);
Load_Image_Bitmap(&ending1,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/ENDING2.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&ending2, 0,0, 640, 480);
Load_Image_Bitmap(&ending2,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// load the background
Load_Bitmap_File(&bitmap8bit, "IMAGE/CREDITS.BMP");

// create and load the level1 bitmap image
Create_Bitmap(&credits, 0,0, 640, 480);
Load_Image_Bitmap(&credits,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

//Load enemy BOB images and animations///////////////////////////////////

// load the explosion bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/EXPLODE2.BMP");

// create explosion bob
Create_BOB(&explosion[0],0,0, 56,72,32, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

Set_Anim_Speed_BOB(&explosion[0], 2);

// load the explosion frames in 
for (int index=0; index < 5; index++)
	Load_Frame_BOB(&explosion[0], &bitmap8bit, index, index, 0, BITMAP_EXTRACT_MODE_CELL);

for (int index=1; index<MAX_EXPLOSIONS; index++)
{
    Clone_BOB(&explosion[0], &explosion[index]);
}

Unload_Bitmap_File(&bitmap8bit);

// load the fire ship bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/ENEMY2.BMP");

// create fire ship bob
Create_BOB(&fire_ship,-56,0, 56,72,32, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the fire ship in 
Load_Frame_BOB(&fire_ship, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload fire ship
Unload_Bitmap_File(&bitmap8bit);

fire_ship.counter_1 = BOSS_HP;

// load the ice ship bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/ENEMY3.BMP");

// create ice ship bob
Create_BOB(&ice_ship,SCREEN_WIDTH,72, 56,72,32, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the ice ship in 
Load_Frame_BOB(&ice_ship, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload ice ship
Unload_Bitmap_File(&bitmap8bit);

ice_ship.counter_1 = BOSS_HP;
ice_ship.counter_2 = 0; //boundary tester--if enemy is at the right border, then set to 1

// load the rocket bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/ROCKET.BMP");

// create rocket bob
Create_BOB(&rocket,300,0, 16,16,5, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the rocket in 
Load_Frame_BOB(&rocket, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload rocket
Unload_Bitmap_File(&bitmap8bit);

// load the icicle bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/ICICLE.BMP");

// create icicle bob
Create_BOB(&icicle,300,0, 16,16,5, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the icicle in 
Load_Frame_BOB(&icicle, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload icicle
Unload_Bitmap_File(&bitmap8bit);

// load the virus bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/VIRUS.BMP");

// create virus bob
Create_BOB(&virus,300,0, 16,16,2, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

Set_Anim_Speed_BOB(&virus, 3);

// load the virus frames in 
for (int index=0; index < 2; index++)
	Load_Frame_BOB(&virus, &bitmap8bit, index, index, 0, BITMAP_EXTRACT_MODE_CELL);

// unload virus
Unload_Bitmap_File(&bitmap8bit);

// load the segment bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/SEGMENT.BMP");

// create segments bob
Create_BOB(&segments[0],300, 72, 56,72,32, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the segments in 
Load_Frame_BOB(&segments[0], &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

segments[0].counter_1 = 100;

for (int num=1; num<NUM_SEGMENTS; num++)
{
    Clone_BOB(&segments[0], &segments[num]);
}

// unload segments
Unload_Bitmap_File(&bitmap8bit);

// load the eyeball bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/EYEBALL.BMP");

// create eyeball bob
Create_BOB(&eyeball,300, 72, 56,72,5, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

Set_Anim_Speed_BOB(&eyeball, 7);

// load the eyeball frames in 
for (int index=0; index < 5; index++)
	Load_Frame_BOB(&eyeball, &bitmap8bit, index, index, 0, BITMAP_EXTRACT_MODE_CELL);

// unload eyeball
Unload_Bitmap_File(&bitmap8bit);

eyeball.counter_1 = BOSS_HP*2;

// load the red crystal bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/RED_CRYSTAL.BMP");

// create red crystal bob
Create_BOB(&crystal1,230,72, 56,72,32, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the red crystal in 
Load_Frame_BOB(&crystal1, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload red crystal
Unload_Bitmap_File(&bitmap8bit);

crystal1.counter_1 = BOSS_HP;
crystal1.anim_state = false;

// load the blue crystal bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/BLUE_CRYSTAL.BMP");

// create blue crystal bob
Create_BOB(&crystal2,370,72, 56,72,32, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the blue crystal in 
Load_Frame_BOB(&crystal2, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload blue crystal
Unload_Bitmap_File(&bitmap8bit);

crystal2.counter_1 = BOSS_HP;
crystal2.anim_state = false;

// load the green crystal bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/GREEN_CRYSTAL.BMP");

// create green crystal bob
Create_BOB(&crystal3,300,144, 56,72,32, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the green crystal in 
Load_Frame_BOB(&crystal3, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload green crystal
Unload_Bitmap_File(&bitmap8bit);

crystal3.counter_1 = BOSS_HP;
crystal3.anim_state = false;

// load the blue fish bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/FISH.BMP");

// create blue fish bob
Create_BOB(&blue_fish[0],300, 72, 56,72,2, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

Set_Anim_Speed_BOB(&blue_fish[0], 3);
blue_fish[0].anim_state = true;
blue_fish[0].counter_1 = NORMAL_HP;

// load the blue fish frames in 
for (int index=0; index < 2; index++)
	Load_Frame_BOB(&blue_fish[0], &bitmap8bit, index, index, 0, BITMAP_EXTRACT_MODE_CELL);

for (int index=1; index<NUM_FISH; index++)
{
    Clone_BOB(&blue_fish[0], &blue_fish[index]);
}

int spacing = 56; //spread the fish apart across the screen
for (int index = 0; index < NUM_FISH; index++)
{
	Set_Pos_BOB(&blue_fish[index], spacing, -(rand()%200)); //random y position
	spacing += 56; //fixed x position
}

// unload blue fish
Unload_Bitmap_File(&bitmap8bit);

// load the red fish bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/FISH2.BMP");

// create red fish bob
Create_BOB(&red_fish[0],300, 72, 56,72,2, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

Set_Anim_Speed_BOB(&red_fish[0], 3);
red_fish[0].anim_state = false;
red_fish[0].counter_1 = NORMAL_HP;

// load the red fish frames in 
for (int index=0; index < 2; index++)
	Load_Frame_BOB(&red_fish[0], &bitmap8bit, index, index, 0, BITMAP_EXTRACT_MODE_CELL);

for (int index=1; index<NUM_FISH; index++)
{
    Clone_BOB(&red_fish[0], &red_fish[index]);
}

spacing = 56; //same as blue fish, fixed x position, random y position
for (int index = 0; index < NUM_FISH; index++)
{
	Set_Pos_BOB(&red_fish[index], spacing, -(rand()%200));
	spacing += 56;
}

// unload red fish
Unload_Bitmap_File(&bitmap8bit);

// load the normal ship bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/ENEMY4.BMP");

// create normal ship bob
Create_BOB(&normal_ship[0],300, 0, 56,72,2, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

Load_Frame_BOB(&normal_ship[0], &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

normal_ship[0].anim_state = false;
normal_ship[0].counter_1 = NORMAL_HP;

for (int index=1; index<NUM_SHIP; index++)
{
    Clone_BOB(&normal_ship[0], &normal_ship[index]);
}

spacing = 56; //same as blue and red fish
for (int index = 0; index < NUM_SHIP; index++)
{
	Set_Pos_BOB(&normal_ship[index], spacing, -(rand()%200));
	spacing += 56;
}

// unload normal ship
Unload_Bitmap_File(&bitmap8bit);

// load the squid bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/SQUID.BMP");

// create squid bob
Create_BOB(&squid[0],300, 0, 56,72,2, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the squid frames in 
for (int index=0; index < 2; index++)
	Load_Frame_BOB(&squid[0], &bitmap8bit, index, index, 0, BITMAP_EXTRACT_MODE_CELL);

squid[0].anim_state = false;
squid[0].counter_1 = NORMAL_HP*4;

for (int index=1; index<NUM_SQUID; index++)
{
    Clone_BOB(&squid[0], &squid[index]);
}

spacing = 56; //same as blue and red fish
for (int index = 0; index < NUM_SQUID; index++)
{
	Set_Pos_BOB(&squid[index], spacing, -(rand()%200));
	spacing += 56;
}

// unload squid
Unload_Bitmap_File(&bitmap8bit);

//Load Player weapon BOB images////////////////////////////////////

// load the normal shot bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/NORMAL_SHOT.BMP");

// create normal shot bob
Create_BOB(&normal_shot,0,0, 16,16,5, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the normal shot in 
Load_Frame_BOB(&normal_shot, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload normal shot
Unload_Bitmap_File(&bitmap8bit);

// load the heat wave bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/HEAT_WAVE3.BMP");

// create heat wave bob
Create_BOB(&heat_wave,0,0, 16,16,5, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the heat wave in 
Load_Frame_BOB(&heat_wave, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload heat wave
Unload_Bitmap_File(&bitmap8bit);

// load the ice pillar bitmaps
Load_Bitmap_File(&bitmap8bit, "IMAGE/ICE_PILLAR3.BMP");

// create ice pillar bob
Create_BOB(&ice_pillar,0,0, 16,16,5, BOB_ATTR_MULTI_FRAME | BOB_ATTR_VISIBLE, 
		   DDSCAPS_SYSTEMMEMORY);

// load the ice pillar in 
Load_Frame_BOB(&ice_pillar, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

// unload ice pillar
Unload_Bitmap_File(&bitmap8bit);

//Load player ship image///////////////////////////////////////////////

if (!Create_BOB(&ship,0,0,56,72,32,
           BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
   return(0);

// build up file name
sprintf(filename,"IMAGE/PLAYER2.BMP");

// load in new bitmap file
Load_Bitmap_File(&bitmap8bit,filename);
 
Load_Frame_BOB(&ship,&bitmap8bit,0,0,0,BITMAP_EXTRACT_MODE_CELL);   

// unload the bitmap file
Unload_Bitmap_File(&bitmap8bit);

ship.counter_1 = 10; //ship HP

// set up stating state of ship
Set_Vel_BOB(&ship, 0,0);
Set_Pos_BOB(&ship, 300, 343);

Init_Projectiles(); //initialize all projectiles

//Load Sound effect files///////////////////////////////////////////////

// load background sounds
shield = DSound_Load_WAV("AUDIO/LOWSHIELD.WAV");

// load background sounds
continuesound = DSound_Load_WAV("AUDIO/CONTINUE.WAV");

// load background sounds
laser_ids[0] = DSound_Load_WAV("AUDIO/LASER.WAV");

// clone sounds
for (int counter=1; counter < NUM_ICICLES; counter++)
    laser_ids[counter] = DSound_Replicate_Sound(laser_ids[0]);

// load background sounds
rocket_ids[0] = DSound_Load_WAV("AUDIO/ROCKETSHOT.WAV");

// clone sounds
for (int counter=1; counter < NUM_ROCKETS; counter++)
    rocket_ids[counter] = DSound_Replicate_Sound(rocket_ids[0]);

// load background sounds
generate_ids[0] = DSound_Load_WAV("AUDIO/GENERATE.WAV");

// clone sounds
for (int counter=1; counter < NUM_VIRUSES; counter++)
    generate_ids[counter] = DSound_Replicate_Sound(generate_ids[0]);

// load background sounds
beam_ids[0] = DSound_Load_WAV("AUDIO/BEAM.WAV");

// clone sounds
for (int counter=1; counter < NUM_SHOTS; counter++)
    beam_ids[counter] = DSound_Replicate_Sound(beam_ids[0]);

// load background sounds
ice_ids[0] = DSound_Load_WAV("AUDIO/ICE.WAV");

// clone sounds
for (int counter=1; counter < NUM_SHOTS; counter++)
    ice_ids[counter] = DSound_Replicate_Sound(ice_ids[0]);

// load background sounds
fire_ids[0] = DSound_Load_WAV("AUDIO/FIRE.WAV");

// clone sounds
for (int counter=1; counter < NUM_SHOTS; counter++)
    fire_ids[counter] = DSound_Replicate_Sound(fire_ids[0]);

// load background sounds
explode_ids[0] = DSound_Load_WAV("AUDIO/EXPLODE.WAV");

// clone sounds
for (int counter=1; counter < MAX_EXPLOSIONS; counter++)
    explode_ids[counter] = DSound_Replicate_Sound(explode_ids[0]);

// set clipping rectangle to screen extents so mouse cursor
// doens't mess up at edges
RECT screen_rect = {0,0,screen_width,screen_height};
lpddclipper = DDraw_Attach_Clipper(lpddsback,1,&screen_rect);

// hide the mouse
ShowCursor(FALSE);

// return success
return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms,  int num_parms)
{
// this function is where you shutdown your game and
// release all resources that you allocated

//delete BOB arrays
Delete_Segments();
Delete_Explosions();
Delete_Blue_Fishes();
Delete_Red_Fishes();
Delete_Normal_Ship();
Delete_Squids();

//delete background images
Destroy_Bitmap(&level1);
Destroy_Bitmap(&level2);
Destroy_Bitmap(&titlescreen);
Destroy_Bitmap(&gameoverscreen);
Destroy_Bitmap(&keyboard);
Destroy_Bitmap(&story1);
Destroy_Bitmap(&story2);
Destroy_Bitmap(&story3);
Destroy_Bitmap(&story4);
Destroy_Bitmap(&story5);
Destroy_Bitmap(&directions);
Destroy_Bitmap(&ending1);
Destroy_Bitmap(&ending2);
Destroy_Bitmap(&credits);

//delete enemy BOBs
Destroy_BOB(&fire_ship);
Destroy_BOB(&rocket);
Destroy_BOB(&icicle);
Destroy_BOB(&virus);
Destroy_BOB(&ice_ship);
Destroy_BOB(&eyeball);
Destroy_BOB(&crystal1);
Destroy_BOB(&crystal2);
Destroy_BOB(&crystal3);

//delete ship and weapon BOBs
Destroy_BOB(&ship);
Destroy_BOB(&heat_wave);
Destroy_BOB(&ice_pillar);
Destroy_BOB(&normal_shot);

// release keyboard
lpdikey->Unacquire();
lpdikey->Release();
lpdi->Release();

// delete all the music
DMusic_Delete_All_MIDI();

// shutdown directmusic
DMusic_Shutdown();

// now directsound
DSound_Stop_All_Sounds();
DSound_Shutdown();

// shutdonw directdraw
DDraw_Shutdown();

// return success
return(1);
} // end Game_Shutdown

///////////////////////////////////////////////////////////

//Sound Generation functions///////////////////////////////

void Explosion_Sound(void)
{        
// this functions hunts for an open handle to play an explosion sound

	// start a hit sound
	for (int sound_index=0; sound_index < MAX_EXPLOSIONS; sound_index++)
	{
		// test if this sound is playing
		if (DSound_Status_Sound(explode_ids[sound_index])==0)
		{
			DSound_Play(explode_ids[sound_index]);
			break;
		} // end if

    } // end for 

} // end Explosion_Sound

void Beam_Sound(void)
{        
// this functions hunts for an open handle to play a normal shot sound

	// start a hit sound
	for (int sound_index=0; sound_index < NUM_SHOTS; sound_index++)
    {
		// test if this sound is playing
		if (DSound_Status_Sound(beam_ids[sound_index])==0)
		{
			DSound_Play(beam_ids[sound_index]);
			break;
		} // end if

    } // end for 

} // end Beam_Sound

void Ice_Sound(void)
{        
// this functions hunts for an open handle to play an ice shot sound

	// start a hit sound
	for (int sound_index=0; sound_index < NUM_SHOTS; sound_index++)
    {
		// test if this sound is playing
		if (DSound_Status_Sound(ice_ids[sound_index])==0)
		{
			DSound_Play(ice_ids[sound_index]);
			break;
		} // end if

    } // end for 

} // end Ice_Sound


void Fire_Sound(void)
{        
// this functions hunts for an open handle to play a heat wave sound

	// start a hit sound
	for (int sound_index=0; sound_index < NUM_SHOTS; sound_index++)
    {
		// test if this sound is playing
		if (DSound_Status_Sound(fire_ids[sound_index])==0)
		{
			DSound_Play(fire_ids[sound_index]);
			break;
		} // end if

    } // end for 

} // end Fire_Sound

void Rocket_Sound(void)
{        
// this functions hunts for an open handle to play a rocket sound

	// start a hit sound
	for (int sound_index=0; sound_index < NUM_ROCKETS; sound_index++)
    {
		// test if this sound is playing
		if (DSound_Status_Sound(rocket_ids[sound_index])==0)
		{
			DSound_Play(rocket_ids[sound_index]);
			break;
		} // end if

    } // end for 

} // end Rocket_Sound

void Laser_Sound(void)
{        
// this functions hunts for an open handle to play a laser sound

	// start a hit sound
	for (int sound_index=0; sound_index < NUM_ICICLES; sound_index++)
    {
		// test if this sound is playing
		if (DSound_Status_Sound(laser_ids[sound_index])==0)
		{
			DSound_Play(laser_ids[sound_index]);
			break;
		} // end if

    } // end for 

} // end Laser_Sound

void Generate_Sound(void)
{        
// this functions hunts for an open handle to play a virus sound

	// start a hit sound
	for (int sound_index=0; sound_index < NUM_VIRUSES; sound_index++)
    {
		// test if this sound is playing
		if (DSound_Status_Sound(generate_ids[sound_index])==0)
		{
			DSound_Play(generate_ids[sound_index]);
			break;
		} // end if

    } // end for 

} // end Generate_Sound

//Enemey AI functions//////////////////////////////////////////

void Eyeball_AI(void)
{
	bool hit;

	Check_Player_Enemy_Collision(eyeball, hit);

	if (!ship_defeated) //don't want the eyeball to follow ship all
	{					//the way to the player's starting position
		// first x-axis 
		if (ship.x > eyeball.x)
			eyeball.x+=(NUM_SEGMENTS+1);
		else if (ship.x < eyeball.x)
			eyeball.x-=(NUM_SEGMENTS+1);

		// now y-axis
		if (ship.y > eyeball.y)
			eyeball.y+=(NUM_SEGMENTS+1);
		else if (ship.y < eyeball.y)
			eyeball.y-=(NUM_SEGMENTS+1);
	}
	else //player defeated, go to the eyeball's spot of origin
	{
		// first x-axis 
		if (300 > eyeball.x)
			eyeball.x+=(NUM_SEGMENTS+1);
		else if (300 < eyeball.x)
			eyeball.x-=(NUM_SEGMENTS+1);

		// now y-axis
		if (72 > eyeball.y)
			eyeball.y+=(NUM_SEGMENTS+1);
		else if (72 < eyeball.y)
			eyeball.y-=(NUM_SEGMENTS+1);

		if (eyeball.x == 300 && eyeball.y == 72) //chase after ship again
			ship_defeated = false;
	}

	// check boundaries
	if (eyeball.x >= SCREEN_WIDTH)
		eyeball.x = -eyeball.width;
	else if (eyeball.x < -eyeball.width)
		eyeball.x = SCREEN_WIDTH;

	if (eyeball.y >= SCREEN_HEIGHT)
		eyeball.y = -eyeball.height;
	else if (eyeball.y < -eyeball.height)
		eyeball.y = SCREEN_HEIGHT;
}

void Segment_AI(void)
{
	bool hit;

	for (int index = 0; index < NUM_SEGMENTS; index++)
	{
		if (segments[index].anim_state == false) //segment destroyed
			for (int num = index; num < NUM_SEGMENTS; num++)
			{
				if (segments[num].anim_state) //if still animated
				{ //destroy it
					Explode_Area(segments[num].x,segments[num].y, 
								 EXPLODE_BIG_X,EXPLODE_BIG_Y);
					segments[num].x = OFFSCREEN_X; segments[num].y = OFFSCREEN_Y;
					segments[num].anim_state = false;
				}
			}
		else //segment is alive
		{
			Check_Player_Enemy_Collision(segments[index], hit);
			if (index == 0) //1st segment follows eyeball
			{
				// first x-axis 
				if (eyeball.x > segments[index].x)
					segments[index].x+=NUM_SEGMENTS;
				else if (eyeball.x < segments[index].x)
					segments[index].x-=NUM_SEGMENTS;

				// now y-axis
				if (eyeball.y > segments[index].y)
					segments[index].y+=NUM_SEGMENTS;
				else if (eyeball.y < segments[index].y)
					segments[index].y-=NUM_SEGMENTS;
			}
			else //follow previous segment
			{
				// first x-axis 
				if (segments[index-1].x > segments[index].x)
					segments[index].x+=(NUM_SEGMENTS-index);
				else if (segments[index-1].x < segments[index].x)
					segments[index].x-=(NUM_SEGMENTS-index);

				// now y-axis
				if (segments[index-1].y > segments[index].y)
					segments[index].y+=(NUM_SEGMENTS-index);
				else if (segments[index-1].y < segments[index].y)
					segments[index].y-=(NUM_SEGMENTS-index);
			}

			// check boundaries
			if (segments[index].x >= SCREEN_WIDTH)
				segments[index].x = -segments[index].width;
			else if (segments[index].x < -segments[index].width)
				segments[index].x = SCREEN_WIDTH;

			if (segments[index].y >= SCREEN_HEIGHT)
				segments[index].y = -segments[index].height;
			else if (segments[index].y < -segments[index].height)
				segments[index].y = SCREEN_HEIGHT;
		}
	}
}

void Blue_fish_AI(void)
{
	bool hit = false; //flag to see if fish collide with ship

	for (int index = 0; index < NUM_FISH; index++)
	{
		Check_Player_Enemy_Collision(blue_fish[index], hit);
		if (hit) //if collide with ship, destroy fish too
		{
			blue_fish[index].x = OFFSCREEN_X; blue_fish[index].y = OFFSCREEN_Y;
			blue_fish[index].anim_state = false;
		}
		hit = false; //reset flag
	    blue_fish[index].y+= (rand()%5+1); //set a random vertical speed
		if (blue_fish[index].y+72 >= MAX_SCREEN_HEIGHT && blue_fish[index].anim_state)
		{//destroy fish if they hit the scoreboard (don't want them to swim through it)
			Explosion_Sound();
			Explode_Area(blue_fish[index].x, blue_fish[index].y, 
						 EXPLODE_BIG_X, EXPLODE_BIG_Y);
			blue_fish[index].x = OFFSCREEN_X; blue_fish[index].y = OFFSCREEN_Y;
	        blue_fish[index].anim_state = false;
		}
	}
}

void Red_fish_AI(void)
{
	bool hit = false; //flag to see if fish collide with ship

	for (int index = 0; index < NUM_FISH; index++)
	{
		Check_Player_Enemy_Collision(red_fish[index], hit);
		if (hit) //if collide with ship, destroy fish too
		{
			red_fish[index].x = OFFSCREEN_X; red_fish[index].y = OFFSCREEN_Y;
			red_fish[index].anim_state = false;
		}
		hit = false; //reset flag
	    red_fish[index].y+= (rand()%7+1); //set a random speed faster than blue fish
		if (red_fish[index].y+72 >= MAX_SCREEN_HEIGHT && red_fish[index].anim_state)
		{//destroy fish if they hit the scoreboard
			Explosion_Sound();
			Explode_Area(red_fish[index].x, red_fish[index].y, EXPLODE_BIG_X, EXPLODE_BIG_Y);
			red_fish[index].x = OFFSCREEN_X; red_fish[index].y = OFFSCREEN_Y;
	        red_fish[index].anim_state = false;
		}
	}
}

void Normal_ship_AI(void)
{
	bool hit = false; //flag to see if normal ship collided with ship

	for (int index = 0; index < NUM_SHIP; index++)
	{
		Check_Player_Enemy_Collision(normal_ship[index], hit);
		if (hit) //if collide with ship, destroy normal ship too
		{
			normal_ship[index].x = OFFSCREEN_X; normal_ship[index].y = OFFSCREEN_Y;
			normal_ship[index].anim_state = false;
		}
		hit = false; //reset flag
		//set random speeds
	    // first x-axis 
		if (ship.x > normal_ship[index].x) //act like homing missile onto ship
			normal_ship[index].x+=(rand()%7+1);
		else
		if (ship.x < normal_ship[index].x)
			normal_ship[index].x-=(rand()%7+1);

		normal_ship[index].y += (rand()%7+1); //move forward, don't go back
		if (normal_ship[index].y+72 >= MAX_SCREEN_HEIGHT && normal_ship[index].anim_state)
		{//destroy normal ship if they hit the scoreboard
			Explosion_Sound();
			Explode_Area(normal_ship[index].x, normal_ship[index].y, 
						 EXPLODE_BIG_X, EXPLODE_BIG_Y);
			normal_ship[index].x = OFFSCREEN_X; normal_ship[index].y = OFFSCREEN_Y;
	        normal_ship[index].anim_state = false;
		}
	}
}

void Squid_AI(void)
{
	bool hit = false; //flag to see if squid collided with ship
	int number; //for setting random numbers

	for (int index = 0; index < NUM_SQUID; index++)
	{
		Check_Player_Enemy_Collision(squid[index], hit);
		if (hit) //if collide with ship, destroy squid too
		{
			squid[index].x = OFFSCREEN_X; squid[index].y = OFFSCREEN_Y;
			squid[index].anim_state = false;
		}
		hit = false; //reset flag
		number = rand()%2; //determines whether squid moves left or right
		if (number == 0)
			squid[index].x += (rand()%5);
		else
			squid[index].x -= (rand()%5);
	    squid[index].y+= (rand()%2+1); //slow and random vertical speed
		if (squid[index].y+72 >= MAX_SCREEN_HEIGHT && squid[index].anim_state)
		{//destroy squid if they hit the scoreboard
			Explosion_Sound();
			Explode_Area(squid[index].x, squid[index].y, EXPLODE_BIG_X, EXPLODE_BIG_Y);
			squid[index].x = OFFSCREEN_X; squid[index].y = OFFSCREEN_Y;
	        squid[index].anim_state = false;
		}
	}
}

void Fire_ship_AI(void)
{
	// this function performs the fire ship ai
	bool hit;
	
	Check_Player_Enemy_Collision(fire_ship, hit);
	Fire_Enemy1_Projectile(90, 5);

	// do tracking algorithm
	// first x-axis 
	if (ship.x > fire_ship.x)
		fire_ship.x+=2;
	else if (ship.x < fire_ship.x)
		fire_ship.x-=2;

	// check boundaries
	if (fire_ship.x >= SCREEN_WIDTH)
		fire_ship.x = -fire_ship.width;
	else if (fire_ship.x < -fire_ship.width)
		fire_ship.x = SCREEN_WIDTH;

	if (fire_ship.y >= SCREEN_HEIGHT)
		fire_ship.y = -fire_ship.height;
	else if (fire_ship.y < -fire_ship.height)
		fire_ship.y = SCREEN_HEIGHT;
} // end Fire_ship_AI

void Ice_Ship_AI(void)
{
	// this function performs the ice ship ai
	bool hit;

	//only fire if directly above ship
	if (ice_ship.x + 10 >= ship.x && ice_ship.x - 10 <= ship.x)
		Fire_Enemy2_Projectile(90, 20);

	Check_Player_Enemy_Collision(ice_ship, hit);

	// check boundaries
	if (ice_ship.x+56 >= SCREEN_WIDTH) //reached right of the screen
		ice_ship.counter_2 = 1;
	else if (ice_ship.x-56 < -ice_ship.width) //reached left of the screen
		ice_ship.counter_2 = 0;

	// do tracking algorithm
	// first x-axis 
	if (ice_ship.counter_2 == 0) //move right
		ice_ship.x+=5;
	else //move left
		ice_ship.x-=5;
} // end Ice_Ship_AI

//Projectile functions////////////////////////////////////////

void Init_Projectiles(void)
{
	// this function initializes the projectiles
	memset(missiles, 0, sizeof(PROJECTILE)*NUM_SHOTS);
	memset(rockets, 0, sizeof(PROJECTILE)*NUM_ROCKETS);
	memset(viruses, 0, sizeof(PROJECTILE)*NUM_VIRUSES);
} // end Init_Projectiles

void Move_Projectiles(void)
{
	// this function moves all the projectiles and does the physics model
	int index;

	for (index=0; index<NUM_SHOTS; index++)
    {
		if (missiles[index].state==1)
		{
			missiles[index].y+=missiles[index].yv;

			// test for off screen
			if (missiles[index].x >= screen_width || 
				missiles[index].y >= MAX_SCREEN_HEIGHT || 
				missiles[index].y < 0) 
			{
				// kill the missile
				missiles[index].state = 0;
			} // end if
        } // end if on
		//check for damage on any possible enemy
		for (int num = 0; num < NUM_FISH; num++)
		    Check_Damage(blue_fish[num], missiles[index], 2);
		for (int num = 0; num < NUM_FISH; num++)
		    Check_Damage(red_fish[num], missiles[index], 1);
		for (int num = 0; num < NUM_SHIP; num++)
		    Check_Damage(normal_ship[num], missiles[index], 0);
		for (int num = 0; num < NUM_SQUID; num++)
		    Check_Damage(squid[num], missiles[index], 2);
		Check_Damage(fire_ship, missiles[index], 1);
		Check_Damage(ice_ship, missiles[index], 2);
		Check_Damage(eyeball, missiles[index], 0);
		Check_Damage(crystal1, missiles[index], 1);
		Check_Damage(crystal2, missiles[index], 2);
		Check_Damage(crystal3, missiles[index], 0);
		for (int num = 0; num < NUM_SEGMENTS; num++)
		    Check_Damage(segments[num], missiles[index], 1);
    } // end for index

	//Projectile data for enemy ships
	for (index=0; index<NUM_ROCKETS; index++)
    {
		if (rockets[index].state==1)
		{
			if (ship.x > rockets[index].x-25) //homing rockets
				rockets[index].x+=6;
			else if (ship.x < rockets[index].x-25)
				rockets[index].x-=6;
			rockets[index].y+=rockets[index].yv;

        // test for off screen
			if (rockets[index].x >= screen_width || 
				rockets[index].y >=MAX_SCREEN_HEIGHT/*screen_height*/ || 
				rockets[index].y < 0) 
			{
				// kill the missile
				rockets[index].x = OFFSCREEN_X; rockets[index].y = OFFSCREEN_Y;
				rockets[index].state = 0;
			} // end if

        } // end if on
		Check_Player_Damage(rockets[index]); //see if rocket hits ship
	}

	//Projectile data for enemy ships
	for (index=0; index<NUM_ICICLES; index++)
    {
		if (icicles[index].state==1)
		{
			icicles[index].y+=icicles[index].yv;

			// test for off screen
			if (icicles[index].x >= screen_width || 
				icicles[index].y >= MAX_SCREEN_HEIGHT/*screen_height*/ || 
				icicles[index].y < 0) 
			{
				// kill the missile
				icicles[index].x = OFFSCREEN_X; icicles[index].y = OFFSCREEN_Y;
				icicles[index].state = 0;
			} // end if

        } // end if on
		Check_Player_Damage(icicles[index]);
	}

	//Projectile data for enemy ships
	for (index=0; index<NUM_VIRUSES; index++)
    {
		if (viruses[index].state==1)
		{
			if (ship.x > viruses[index].x-25) //homing viruses
				viruses[index].x+=rand()%5;
			else if (ship.x < viruses[index].x-25)
				viruses[index].x-=rand()%5;

			if (ship.y > viruses[index].y-25) //homing viruses
				viruses[index].y+=rand()%5;
			else if (ship.y < viruses[index].x-25)
				viruses[index].y-=rand()%5;

			// test for off screen
			if (viruses[index].x >= screen_width || 
				viruses[index].y >= MAX_SCREEN_HEIGHT/*screen_height*/ || 
				viruses[index].y < 0) 
			{
				// kill the missile
				viruses[index].state = 0;
			} // end if

        } // end if on
		Check_Player_Damage(viruses[index]);
	}
} // end Move_Projectiles

/////////////////////////////////////////////////////////////

void Draw_Projectiles(void)
{
	int color, index;
	// this function draws all the projectiles 
	for (index=0; index < NUM_SHOTS; index++)
    {
		// is this one alive?
		if (missiles[index].state==1)
		{
			switch (weapon)
			{
			case 0:
				normal_shot.x = missiles[index].x; normal_shot.y = missiles[index].y;
				Draw_BOB(&normal_shot, lpddsback); break;
			case 1:
				ice_pillar.x = missiles[index].x; ice_pillar.y = missiles[index].y;
				Draw_BOB(&ice_pillar, lpddsback); break;
			case 2:
				heat_wave.x = missiles[index].x; heat_wave.y = missiles[index].y;
				Draw_BOB(&heat_wave, lpddsback); break;
			default: color = 0;
			}
       } // end if
    } // end for index

	for (index=0; index < NUM_ROCKETS; index++)
    {
		// is this one alive?
		if (rockets[index].state==1)
		{
			rocket.x = rockets[index].x;
			rocket.y = rockets[index].y;
			Draw_BOB(&rocket, lpddsback);
		} // end if
    } // end for index

	for (index=0; index < NUM_ICICLES; index++)
    {
		// is this one alive?
		if (icicles[index].state==1)
		{
			icicle.x = icicles[index].x;
			icicle.y = icicles[index].y;
			Draw_BOB(&icicle, lpddsback);
		} // end if
    } // end for index

	for (index=0; index < NUM_VIRUSES; index++)
    {
		// is this one alive?
		if (viruses[index].state==1)
		{
			virus.x = viruses[index].x;
			virus.y = viruses[index].y;
			Animate_BOB(&virus);
			Draw_BOB(&virus, lpddsback);
		} // end if
    } // end for index
} // end Draw_Projectiles

void Fire_Projectile(int angle, float vel)
{
	// this function starts a projectile with the given velocity 
	// at the tip of the player ship
	for (int index=0; index < NUM_SHOTS; index++)
    {
		// find an open projectile
		if (missiles[index].state==0)
		{
			// set this missile in motion at the head of ship
			missiles[index].x = ship.x+21;
			missiles[index].y = ship.y;
			missiles[index].xv = 0;
			missiles[index].yv = -vel;

			// mark as active
			missiles[index].state = 1;

			// make sound
			if (weapon == 0)
				Beam_Sound();
			else if (weapon == 1)
				Ice_Sound();
			else if (weapon == 2)
				Fire_Sound();
			// bail
			break;
       } // end if
   } // end for index
} // end Fire_Projectile

void Fire_Enemy1_Projectile(int angle, float vel)
{
	// this function starts a projectile with the given velocity 
	// at the tip of the fire ship
	for (int index=0; index < NUM_ROCKETS; index++)
    {
		// find an open projectile
		if (rockets[index].state==0)
		{
			// set this missile in motion at the head of fire ship
			rockets[index].x = fire_ship.x+30;
			rockets[index].y = fire_ship.y+50;
			rockets[index].xv = 0; 
			rockets[index].yv = vel;

			// mark as active
			rockets[index].state = 1;

			// make sound
			Rocket_Sound();

			// bail
			break;
		} // end if
	} // end for index
} // end Fire_Projectile

void Fire_Enemy2_Projectile(int angle, float vel)
{
	// this function starts a projectile with the given velocity 
	// at the tip of the ice ship
	for (int index=0; index < NUM_ICICLES; index++)
    {
		// find an open projectile
		if (icicles[index].state==0)
		{
			// set this missile in motion at the head of ice ship
			icicles[index].x = ice_ship.x+30;
			icicles[index].y = ice_ship.y+50;
			icicles[index].xv = 0; 
			icicles[index].yv = vel;

			// mark as active
			icicles[index].state = 1;

			// make sound
			Laser_Sound();

			// bail
			break;
		} // end if
	} // end for index
} // end Fire_Projectile

void Fire_Enemy3_Projectile(int angle, float vel)
{
	// this function starts a projectile with the given angle and velocity 
	// at the center of the eyeball
	for (int index=0; index < NUM_VIRUSES; index++)
    {
		// find an open projectile
		if (viruses[index].state==0)
		{
			// set this missile in motion at the center of the eyeball
			viruses[index].x = eyeball.x+30;
			viruses[index].y = eyeball.y+30;
			viruses[index].xv = 0; 
			viruses[index].yv = vel;

			// mark as active
			viruses[index].state = 1;

			// make sound
			Generate_Sound();

			// bail
			break;
		} // end if
	} // end for index
} // end Fire_Projectile

//Damage functions///////////////////////////////////////////////

void Check_Player_Damage(PROJECTILE& missile)
{
	if (missile.x >= ship.x && missile.x <= ship.x+56) //check if collide
		if (missile.y >= ship.y && missile.y <= ship.y+72) //with missile
		{
			ship.counter_1--; //lower ship's shields
			Explode_Area(missile.x, missile.y, EXPLODE_SMALL_X, EXPLODE_SMALL_Y);
			missile.x = OFFSCREEN_X; missile.y = OFFSCREEN_Y;
			if (ship.counter_1 <= 0) //ship defeated
			    Player_Defeated();
		}
}

void Check_Player_Enemy_Collision(BOB enemy, bool &hit)
{//checks if enemy collides with ship
	if (enemy.anim_state == true) //only collide if enemy exists
	if (ship.x >= enemy.x && ship.x <= enemy.x+30 || 
		ship.x+30 >= enemy.x && ship.x+30 <= enemy.x+30)
		if (ship.y >= enemy.y && ship.y <= enemy.y+60 || 
			ship.y+60 >= enemy.y && ship.y+60 <= enemy.y+60)
		{
			ship.counter_1-=10; //destroys ship
			hit = true;
			if (ship.counter_1 <= 0) //ship defeated
			    Player_Defeated();
		}
}

void Check_Damage(BOB& target, PROJECTILE& missile, int weakness)
{
	if (target.anim_state) //only hit if enemy exists
	if (missile.x >= target.x && missile.x <= target.x+56)
		if (missile.y >= target.y && missile.y <= target.y+72)
		{
			if (weakness == weapon) //weakness exploited, do major damage
			    target.counter_1-=5;
			else target.counter_1--; //used weapon not weak against, little damage
			Explode_Area(missile.x, missile.y-20,EXPLODE_SMALL_X,EXPLODE_SMALL_Y);
			missile.x = OFFSCREEN_X; missile.y = OFFSCREEN_Y;
			if (target.counter_1 <= 0) //enemy defeated
			{
				Explosion_Sound();
				Explode_Area(target.x+7, target.y,EXPLODE_BIG_X,EXPLODE_BIG_Y);
				scoreboard += 100; //get 100 points
			    target.anim_state = false;
			    target.x = SCREEN_WIDTH;
			    target.y = 0;
			}
		}
}

void Explode_Area(int x_pos, int y_pos, int x_size, int y_size)
{
	int num;
	for (num = 0; num < MAX_EXPLOSIONS; num++) //checks for available explosion
		if (explosion[num].anim_state == false) break;
	explosion[num].curr_frame = 0; //resets animation
	explosion[num].x = x_pos; //set position
	explosion[num].y = y_pos;
	explosion[num].varsI[0] = x_size; //set size
	explosion[num].varsI[1] = y_size;
	explosion[num].state = 1;
	explosion[num].anim_state = true; //animate explosion
}

void Player_Defeated()
{
	num_ships--; //lower total of available ships
	scoreboard -= 100; //lower score
	ship.counter_1 = 10; //reset shields
	Explode_Area(ship.x, ship.y,EXPLODE_BIG_X,EXPLODE_BIG_Y);
	Explosion_Sound();
	ship_defeated = true; //mark flag for eyeball
	ship.x = 300; ship.y = 343; //starting position
}

//Reset Game////////////////////////////////////////////////////

void ResetGame(void)
{
	// load the background
	Load_Bitmap_File(&bitmap8bit, "IMAGE/LEVEL1.BMP");
	// set the palette to background image palette
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit);
	//global variables for weapon equipped, total ships, scoreboard, and various flags
	num_ships = 5;
	ship.counter_1 = 10;
	points_awarded = 1;
	num_wave = 0; //determines which enemy wave to appear
	scoreboard = 0;
	endingcounter = 0; //tells which ending slide to show

	for (int index = 0; index < NUM_VIRUSES; index++)
	{
		viruses[index].x = OFFSCREEN_X;
		viruses[index].y = OFFSCREEN_Y;
		viruses[index].state = 0;
	}

	for (int index = 0; index < NUM_ROCKETS; index++)
	{
		rockets[index].x = OFFSCREEN_X;
		rockets[index].y = OFFSCREEN_Y;
		rockets[index].state = 0;
	}

	//flags for game control
	ship_defeated = false; //flag for 2nd boss (eyeball)
	gamestart = true; //flag for starting the game
	gameover = false; //game over flag
	gameend = false; //ending flag
	key_space_up = false; //flag for storytelling

	//reset enemy HP and positions
	fire_ship.x = -56; fire_ship.y = 0;
	fire_ship.counter_1 = BOSS_HP;
	fire_ship.anim_state = false;

	ice_ship.x = 72; ice_ship.y = 56;
	ice_ship.counter_1 = BOSS_HP;
	ice_ship.counter_2 = 0;
	ice_ship.anim_state = false;

	for (int num=0; num<NUM_SEGMENTS; num++)
	{
		segments[num].counter_1 = 100;
		segments[num].x = 300; segments[num].y = 72;
		segments[num].anim_state = false;
	}

	eyeball.x = 300; eyeball.y = 72;
	eyeball.counter_1 = BOSS_HP*2;
	eyeball.anim_state = false;

	crystal1.x = 230; crystal1.y = 72;
	crystal1.counter_1 = BOSS_HP;
	crystal1.anim_state = false;

	crystal2.x = 370; crystal2.y = 72;
	crystal2.counter_1 = BOSS_HP;
	crystal2.anim_state = false;

	crystal3.x = 300; crystal3.y = 144;
	crystal3.counter_1 = BOSS_HP;
	crystal3.anim_state = false;

	int spacing = 56; //spread the fish apart across the screen
	for (int index = 0; index < NUM_FISH; index++)
	{
		blue_fish[index].anim_state = true;
		blue_fish[index].counter_1 = NORMAL_HP;
		Set_Pos_BOB(&blue_fish[index], spacing, -(rand()%200)); //random y position
		spacing += 56; //fixed x position
	}

	spacing = 56; //same as blue fish, fixed x position, random y position
	for (int index = 0; index < NUM_FISH; index++)
	{
		red_fish[index].anim_state = false;
		red_fish[index].counter_1 = NORMAL_HP;
		Set_Pos_BOB(&red_fish[index], spacing, -(rand()%200));
		spacing += 56;
	}

	spacing = 56; //same as blue and red fish
	for (int index = 0; index < NUM_SHIP; index++)
	{
		normal_ship[index].anim_state = false;
		normal_ship[index].counter_1 = NORMAL_HP;
		Set_Pos_BOB(&normal_ship[index], spacing, -(rand()%200));
		spacing += 56;
	}

	spacing = 56; //same as blue and red fish
	for (int index = 0; index < NUM_SQUID; index++)
	{
		squid[index].anim_state = false;
		squid[index].counter_1 = NORMAL_HP*4;
		Set_Pos_BOB(&squid[index], spacing, -(rand()%200));
		spacing += 56;
	}
}


//Deletion functions////////////////////////////////////////////

void Delete_Segments(void)
{
	// this function simply deletes all memory and surfaces
	// related to the segments
	int index;
	for (index=0; index < NUM_SEGMENTS; index++)
		Destroy_BOB(&segments[index]);
} // end Delete_Segments

void Delete_Explosions(void)
{
	// this function simply deletes all memory and surfaces
	// related to the explosions
	int index;
	for (index=0; index < MAX_EXPLOSIONS; index++)
		Destroy_BOB(&explosion[index]);
} // end Delete_Explosions

void Delete_Blue_Fishes(void)
{
	// this function simply deletes all memory and surfaces
	// related to the blue fishes
	int index;
	for (index=0; index < NUM_FISH; index++)
		Destroy_BOB(&blue_fish[index]);
} // end Delete_Blue_Fishes

void Delete_Red_Fishes(void)
{
	// this function simply deletes all memory and surfaces
	// related to the red fishes
	int index;
	for (index=0; index < NUM_FISH; index++)
		Destroy_BOB(&red_fish[index]);
} // end Delete_Red_Fishes

void Delete_Normal_Ship(void)
{
	// this function simply deletes all memory and surfaces
	// related to the normal ships
	int index;
	for (index=0; index < NUM_SHIP; index++)
		Destroy_BOB(&normal_ship[index]);
} // end Delete_Normal_Ship

void Delete_Squids(void)
{
	// this function simply deletes all memory and surfaces
	// related to the squids
	int index;
	for (index=0; index < NUM_SQUID; index++)
		Destroy_BOB(&squid[index]);
} // end Delete_Squids

//Start main game///////////////////////////////////////////////

int Game_Main(void *parms, int num_parms)
{
// this is the workhorse of your game it will be called
// continuously in real-time this is like main() in C
// all the calls for you game go here!

int          index;             // looping var
int          dx,dy;             // general deltas used in collision detection
int			 num;
char 		 str[10];				// character string
char 		 ships_left[30];
char		 score[30];
char		 ship_health[40];
 
static int   player_moving = 0; // tracks player motion
static PALETTEENTRY glow = {0,0,0,PC_NOCOLLAPSE};  // used to animation red border
static PALETTEENTRY glow2 = {0,0,0,PC_NOCOLLAPSE};
static int glow_count = 0, glow_dx = 5;
static int glow2_count = 0, glow2_dx = 5;

// check of user is trying to exit
if (KEY_DOWN(VK_ESCAPE))
    PostMessage(main_window_handle, WM_DESTROY,0,0);

// start the timing clock
Start_Clock();

// clear the drawing surface
DDraw_Fill_Surface(lpddsback, 0);

if (!gamestart && !storytime && !gameend) //at the title screen
{
	if (DMusic_Status_MIDI (intromusic) == 3) DMusic_Play(intromusic);

	// lock the back buffer
	DDraw_Lock_Back_Surface();

	// draw the background level1 image
	Draw_Bitmap(&titlescreen, back_buffer, back_lpitch, 0);

	// unlock the back buffer
	DDraw_Unlock_Back_Surface();

	// animate color
	glow.peRed+=glow_dx;

	// test boundary
	if (glow.peRed == 0 || glow.peRed == 255)
		glow_dx = -glow_dx;

	Set_Palette_Entry(WALL_ANIMATION_COLOR, &glow);

	if(KEY_DOWN(VK_SPACE))
	{
		DSound_Play(continuesound);
		// load the background
		Load_Bitmap_File(&bitmap8bit, "IMAGE/STORY1.BMP");
		// set the palette to background image palette
		Set_Palette(bitmap8bit.palette);
		Unload_Bitmap_File(&bitmap8bit);
		storytime = true;
	}
	Draw_Text_GDI("Created by",screen_width/2-50,screen_height/2+15,RGB(0,0,100),lpddsback);
	Draw_Text_GDI("Lawrence Chan",screen_width/2-50,screen_height/2+30,RGB(0,0,100),lpddsback);
	Draw_Text_GDI("Patrick Habib",screen_width/2-50,screen_height/2+45,RGB(0,0,100),lpddsback);
	Draw_Text_GDI("Press the SPACEBAR to start.",250,400,WALL_ANIMATION_COLOR,lpddsback);
	Draw_Text_GDI("Press <ESC> to quit.",250,415,WALL_ANIMATION_COLOR,lpddsback);
}
else if (storytime)
{
	if (DMusic_Status_MIDI (intromusic) == 3) DMusic_Play(intromusic);

    if (KEY_UP(VK_SPACE)) key_space_up = true;

	if (storycounter == 0)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&keyboard, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else if (storycounter == 1)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&story1, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else if (storycounter == 2)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&story2, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else if (storycounter == 3)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&story3, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else if (storycounter == 4)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&story4, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else if (storycounter == 5)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&story5, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else if (storycounter == 6)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&directions, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}

	if (KEY_DOWN(VK_SPACE) && key_space_up)
	{
		DSound_Play(continuesound);
		storycounter++;
		key_space_up = false;
		if (storycounter > 6)
		{
			// load the background
			Load_Bitmap_File(&bitmap8bit, "IMAGE/LEVEL1.BMP");
			// set the palette to background image palette
			Set_Palette(bitmap8bit.palette);
			Unload_Bitmap_File(&bitmap8bit);
			storytime = false;
			gamestart = true;
			if (DMusic_Status_MIDI (intromusic) != 3) DMusic_Stop(intromusic);
		}
	}
}
else if (gamestart && !gameover)
{
	if (num_ships < 0)
	{
		// load the background
		Load_Bitmap_File(&bitmap8bit, "IMAGE/GAMEOVER.BMP");
		// set the palette to background image palette
		Set_Palette(bitmap8bit.palette);
		Unload_Bitmap_File(&bitmap8bit);
		gameover = true;
	}

	if (num_wave < WAVE_CRYSTALS) //load level 1
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&level1, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else //load level 2
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level2 image
		Draw_Bitmap(&level2, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}

	if (crystal1.anim_state || crystal2.anim_state || crystal3.anim_state)
	{//keep doing until all crystals destroyed
		bool hit;

		Animate_BOB(&eyeball);
		Draw_BOB(&eyeball, lpddsback);
		Fire_Enemy3_Projectile(90, 5);

		//check for collision and damage if crystal exists
		if (crystal1.anim_state) 
		{
			Check_Player_Enemy_Collision(crystal1, hit);
			Draw_BOB(&crystal1, lpddsback);
		}
		if (crystal2.anim_state)
		{
			Check_Player_Enemy_Collision(crystal2, hit);
			Draw_BOB(&crystal2, lpddsback);
		}
		if (crystal3.anim_state)
		{
			Check_Player_Enemy_Collision(crystal3, hit);
			Draw_BOB(&crystal3, lpddsback);
		}
	}

	if (eyeball.anim_state)
	{
		// call the eyeball AI
		Eyeball_AI();

		if (eyeball.curr_frame >= eyeball.num_frames-1) 
		{
			eyeball.curr_frame = 0;
		} // end if

		Animate_BOB(&eyeball);
		// draw the eyeball
		Draw_BOB(&eyeball, lpddsback);

		// call the segment AI
		Segment_AI();
		for (int counter = 0; counter < NUM_SEGMENTS; counter++)
			if (segments[counter].anim_state)
				Draw_BOB(&segments[counter], lpddsback);
	}

	if (eyeball.counter_1 <= 0 && num_wave == WAVE_EYEBALL) 
	{//destroy all the extra segments if eyeball defeated
		for (int counter = 0; counter < NUM_SEGMENTS; counter++)
		{
			if (segments[counter].anim_state)
			{
				segments[counter].anim_state = false;
				Explosion_Sound();
				Explode_Area(segments[counter].x,segments[counter].y,
							 EXPLODE_BIG_X,EXPLODE_BIG_Y);
				segments[counter].x = OFFSCREEN_X; segments[counter].y = OFFSCREEN_Y;
			}
		}
		eyeball.anim_state = false;
		num_wave++;
	}

	if (fire_ship.anim_state)
	{
		// call the fire ship AI
		Fire_ship_AI();

		// draw the fire ship
		Draw_BOB(&fire_ship, lpddsback);
	}

	if (ice_ship.anim_state)
	{
		// call the ice ship AI
		Ice_Ship_AI();

		// draw the ice ship
		Draw_BOB(&ice_ship, lpddsback);
	}

	for (int counter = 0; counter < MAX_EXPLOSIONS; counter++)
		if (explosion[counter].anim_state)
		{
			Animate_BOB(&explosion[counter]);
			Draw_Scaled_BOB(&explosion[counter], explosion[counter].varsI[0],
							 explosion[counter].varsI[1], lpddsback);
			if (explosion[counter].curr_frame >= explosion[counter].num_frames-1) 
			{
				explosion[counter].state = 0; explosion[counter].anim_state = false;
			} // end if
		}

	// get player input

	// get the keyboard data
	lpdikey->GetDeviceState(256, (LPVOID)keyboard_state);

	// reset motion flag
	player_moving = 0;

	// test direction of motion, this is a good example of testing the keyboard
	// although the code could be optimized this is more educational

	if(keyboard_state[DIK_SPACE])
	{
		Fire_Projectile(90, wspeed);
	}

	if (keyboard_state[DIK_RIGHT] && keyboard_state[DIK_UP]) 
	{
		// move ship
		ship.x+=10;
		ship.y-=10;
		dx=10; dy=-10;

		// set motion flag
		player_moving = 1;
	} // end if
	else if (keyboard_state[DIK_LEFT] && keyboard_state[DIK_UP]) 
	{
		// move ship
		ship.x-=10;
		ship.y-=10;
		dx=-10; dy=-10;

		// set motion flag
		player_moving = 1;
	} // end if
	else if (keyboard_state[DIK_LEFT] && keyboard_state[DIK_DOWN]) 
	{
		// move ship
		ship.x-=10;
		ship.y+=10;
		dx=-10; dy=10;

		// set motion flag
		player_moving = 1;
	} // end if
	else if (keyboard_state[DIK_RIGHT] && keyboard_state[DIK_DOWN]) 
	{
		// move ship
		ship.x+=10;
		ship.y+=10;
		dx=10; dy=10;

		// set motion flag
		player_moving = 1;
	} // end if
	else if (keyboard_state[DIK_RIGHT]) 
	{
		// move ship
		ship.x+=10;
		dx=10; dy=0;

		// set motion flag
		player_moving = 1;
	} // end if
	else if (keyboard_state[DIK_LEFT])  
	{
		// move ship
		ship.x-=10;
		dx=-10; dy=0; 
   
		// set motion flag
		player_moving = 1;
	} // end if
	else if (keyboard_state[DIK_UP])    
	{
		// move ship
		ship.y-=10;
		dx=0; dy=-10;
   
		// set motion flag
		player_moving = 1;
	} // end if
	else if (keyboard_state[DIK_DOWN])  
	{
		// move ship
		ship.y+=10;
		dx=0; dy=+10;

		// set motion flag
		player_moving = 1;
	} // end if

	if(keyboard_state[DIK_A]) {weapon = 2; wspeed = 5;}
	else if (keyboard_state[DIK_S]) {weapon = 0; wspeed = 10;}
	else if (keyboard_state[DIK_D]) {weapon = 1; wspeed = 15;}

	for (int counter = 1; counter < 10; counter++)
	{//if scoreboard gets incremented by 1000, award player with an extra ship
		if (scoreboard >= (1000 * counter) && points_awarded == counter)
		{
			num_ships++;
			points_awarded++;
		}
	}

	// move all the projectiles
	Move_Projectiles();

	// draw the projectiles
	Draw_Projectiles();

	// only animate if player is moving
	if (player_moving)
	{
		// see if ship hit a wall
		// lock surface, so we can scan it
		DDraw_Lock_Back_Surface();
   
		// try to center the scan in the center of the object to make it 
		// more realistic
		if (Color_Scan(ship.x+16, ship.y+16,
                  ship.x+ship.width-16, ship.y+ship.height-16,                                    
                  WALL_ANIMATION_COLOR, WALL_ANIMATION_COLOR, back_buffer,back_lpitch))
		{
			// back the ship up along its last trajectory
			ship.x-=dx;
			ship.y-=dy;
		} // end if
   
		// done, so unlock
		DDraw_Unlock_Back_Surface();

		// check if ship is off screen
		if (ship.x < 0 || ship.x > (screen_width - ship.width))
			ship.x-=dx;

		if (ship.y < 0 || ship.y > (MAX_SCREEN_HEIGHT - ship.height))
			ship.y-=dy;
	} // end if

	// draw the ship
	Draw_BOB(&ship, lpddsback);

	//game control (determines which wave the player goes through)
	if (num_wave == WAVE_BLUE_FISH)
	{
		bool done = true;
		Blue_fish_AI();
		for (int counter = 0; counter < NUM_FISH; counter++)
		{
			Animate_BOB(&blue_fish[counter]);
			Draw_BOB(&blue_fish[counter], lpddsback);
		}
		for (int counter = 0; counter < NUM_FISH; counter++)
			if (blue_fish[counter].anim_state) done = false;
		if (done)
		{
			for (int counter = 0; counter < NUM_FISH; counter++)
				red_fish[counter].anim_state = true;
			num_wave++;
		}
	}
	else if (num_wave == WAVE_RED_FISH)
	{
		bool done = true;
		Red_fish_AI();
		for (int counter = 0; counter < NUM_FISH; counter++)
		{
			Animate_BOB(&red_fish[counter]);
			Draw_BOB(&red_fish[counter], lpddsback);
		}
		for (int counter = 0; counter < NUM_FISH; counter++)
			if (red_fish[counter].anim_state) done = false;
		if (done)
		{
			for (int counter = 0; counter < NUM_SHIP; counter++)
				normal_ship[counter].anim_state = true;
			num_wave++;
		}
	}
	else if (num_wave == WAVE_NORMAL_SHIP)
	{
		bool done = true;
		Normal_ship_AI();
		for (int counter = 0; counter < NUM_SHIP; counter++)
		{
			if (normal_ship[counter].counter_1 > 0)
				Draw_BOB(&normal_ship[counter], lpddsback);
			else normal_ship[counter].anim_state = false;
		}
		for (int counter = 0; counter < NUM_SHIP; counter++)
			if (normal_ship[counter].anim_state) done = false;
		if (done)
		{
			for (int counter = 0; counter < NUM_SQUID; counter++)
				squid[counter].anim_state = true;
			num_wave++;
		}
	}
	else if (num_wave == WAVE_SQUID)
	{
		bool done = true;
		Squid_AI();
		for (int counter = 0; counter < NUM_SQUID; counter++)
		{
			Animate_BOB(&squid[counter]);
			if (squid[counter].counter_1 > 0)
				Draw_BOB(&squid[counter], lpddsback);
			else squid[counter].anim_state = false;
		}
		for (int counter = 0; counter < NUM_SQUID; counter++)
			if (squid[counter].anim_state) done = false;
		if (done) num_wave++;
	}
	else if (num_wave == WAVE_DUAL_SHIPS)
	{
		if (fire_ship.counter_1 > 0)
			fire_ship.anim_state = true;
		if (ice_ship.counter_1 > 0)
			ice_ship.anim_state = true;
		if (fire_ship.anim_state == false && ice_ship.anim_state == false)
		{
			Draw_Text_GDI("Alien Lair detected.",SCREEN_WIDTH-390,MAX_SCREEN_HEIGHT/2,
						   RGB(0,0,100),lpddsback);
			Draw_Text_GDI("Press the RETURN or ENTER to proceed.",SCREEN_WIDTH-450,
						   MAX_SCREEN_HEIGHT/2+15,RGB(0,0,100),lpddsback);
			if (KEY_DOWN(VK_RETURN))
			{
				ship.x = 300; ship.y = 343;
				num_wave++;
			}
		}
	}
	else if (num_wave == WAVE_CRYSTALS)
	{
		if (crystal1.counter_1 > 0)
			crystal1.anim_state = true;
		if (crystal2.counter_1 > 0)
			crystal2.anim_state = true;
		if (crystal3.counter_1 > 0)
			crystal3.anim_state = true;
		if (!crystal1.anim_state && !crystal2.anim_state && !crystal3.anim_state)
			num_wave++;
	}
	else if (num_wave == WAVE_EYEBALL)
	{
		eyeball.anim_state = true;
		for (int counter = 0; counter < NUM_SEGMENTS; counter++)
			if (segments[counter].counter_1 >0)
				segments[counter].anim_state = true;
	}
	else if (num_wave == ENDING_TIME)
	{
		Draw_Text_GDI("Alien threat annihilated.",SCREEN_WIDTH-390,MAX_SCREEN_HEIGHT/2,
				       RGB(255,150,0),lpddsback);
		Draw_Text_GDI("Press the RETURN or ENTER to proceed.",SCREEN_WIDTH-450,
					   MAX_SCREEN_HEIGHT/2+15,RGB(255,150,0),lpddsback);
		if (KEY_DOWN(VK_RETURN))
		{
			gamestart = false;
			gameend = true;
			Load_Bitmap_File(&bitmap8bit, "IMAGE/ENDING1.BMP");
			// set the palette to background image palette
			Set_Palette(bitmap8bit.palette);
			Unload_Bitmap_File(&bitmap8bit);
		}
	}
	//end game control

	strcpy(ships_left,"Player Ships Left = ");
	itoa(num_ships, str, 10);
	for (num = 0; str[num]; num++);
	str[num] = '\0';
	strcat(ships_left,str);

	strcpy(score,"Player Score = ");
	itoa(scoreboard, str, 10);
	for (num = 0; str[num]; num++);
	str[num] = '\0';
	strcat(score,str);

	strcpy(ship_health,"Ship's Health = ");
	strcpy(str,"||");
	for (num = 0; num < ship.counter_1; num++)
		strcat(ship_health,str);

	if (ship.counter_1 == 1 && DSound_Status_Sound(shield)==0) DSound_Play(shield);
	if (num_wave < WAVE_CRYSTALS)
	{
		if (DMusic_Status_MIDI (lvl1music) == 3)
			DMusic_Play(lvl1music);
	}
	else
	{
		if (DMusic_Status_MIDI (lvl1music) != 3)
			DMusic_Stop(lvl1music);

		if (DMusic_Status_MIDI (lvl2music) == 3)
			DMusic_Play(lvl2music);
	}

	Draw_Text_GDI(ship_health ,10,screen_height - 62,RGB(255,0,0),lpddsback);
	Draw_Text_GDI(ships_left ,10,screen_height - 47,RGB(255,0,0),lpddsback);
	Draw_Text_GDI(score ,10,screen_height - 17,RGB(0,0,255),lpddsback);
	switch(weapon)
	{
	case 0:
	Draw_Text_GDI("Weapon Equipped = Normal Shot",10,screen_height - 32,
				   RGB(0,115,0),lpddsback); break;
	case 1:
	Draw_Text_GDI("Weapon Equipped = Ice Pillar",10,screen_height - 32,
				   RGB(0,0,150),lpddsback); break;
	case 2:
	Draw_Text_GDI("Weapon Equipped = Heat Wave",10,screen_height - 32,
				   RGB(150,0,0),lpddsback); break;
	}

	// draw some text
	Draw_Text_GDI("A Key = Heat Wave",screen_width - 400,screen_height - 62,
				   RGB(0,255,0),lpddsback);
	Draw_Text_GDI("S Key = Normal Shot",screen_width - 400,screen_height - 47,
				   RGB(0,255,0),lpddsback);
	Draw_Text_GDI("D Key = Ice Pillar",screen_width - 400,screen_height - 32,
				   RGB(0,255,0),lpddsback);
	Draw_Text_GDI("SPACEBAR = Shoot Weapon",screen_width - 400,screen_height - 17,
				   RGB(0,255,0),lpddsback);
	Draw_Text_GDI("<ESC> = Quit Game",screen_width - 190,screen_height - 62,
				   RGB(0,255,0),lpddsback);
	Draw_Text_GDI("Arrow Keys = Move Ship",screen_width - 190,screen_height - 47,
				   RGB(0,255,0),lpddsback);
}
else if (gameover)
{
	// lock the back buffer
	DDraw_Lock_Back_Surface();

	// draw the background level1 image
	Draw_Bitmap(&gameoverscreen, back_buffer, back_lpitch, 0);

	// unlock the back buffer
	DDraw_Unlock_Back_Surface();

	if (KEY_DOWN(VK_RETURN))
		ResetGame();
}
else if (gameend)
{
	if (DMusic_Status_MIDI (lvl2music) != 3) DMusic_Stop(lvl2music);
	if (DMusic_Status_MIDI (endingmusic) == 3) DMusic_Play(endingmusic);

    if (KEY_UP(VK_SPACE)) key_space_up = true;

	if (endingcounter == 0)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&ending1, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else if (endingcounter == 1)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&ending2, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
	}
	else if (endingcounter == 2)
	{
		// lock the back buffer
		DDraw_Lock_Back_Surface();

		// draw the background level1 image
		Draw_Bitmap(&credits, back_buffer, back_lpitch, 0);

		// unlock the back buffer
		DDraw_Unlock_Back_Surface();
		if (KEY_DOWN(VK_RETURN))
			ResetGame();
	}
	if (KEY_DOWN(VK_SPACE) && key_space_up && endingcounter < 2)
	{
		DSound_Play(continuesound);
		endingcounter++;
		key_space_up = false;
	}
}

// flip the surfaces
DDraw_Flip();

// sync to 30 fps
Wait_Clock(30);

// return success
return(1);

} // end Game_Main

//////////////////////////////////////////////////////////